//+------------------------------------------------------------------+
//|                                               wz_trail_mm_44.mq5 |
//|                                  Copyright 2024, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include <Expert\Expert.mqh>
//--- available signals
#include <Expert\Signal\My\SignalWZ_44.mqh>
//--- available trailing
#include <Expert\Trailing\My\TrailingWZ_44.mqh>
//--- available money management
#include <Expert\Money\My\MoneyWZ_44.mqh>
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
//--- inputs for expert
input string Expert_Title                = "wz_trail_mm_44"; // Document name
ulong        Expert_MagicNumber          = 24055;           //
bool         Expert_EveryTick            = false;           //
//--- inputs for main signal
input int    Signal_ThresholdOpen        = 10;              // Signal threshold value to open [0...100]
input int    Signal_ThresholdClose       = 10;              // Signal threshold value to close [0...100]
input double Signal_PriceLevel           = 0.0;             // Price level to execute a deal
input double Signal_StopLevel            = 50.0;            // Stop Loss level (in points)
input double Signal_TakeLevel            = 50.0;            // Take Profit level (in points)
input int    Signal_Expiration           = 4;               // Expiration of pending orders (in bars)
input int    Signal_ATR_PeriodMA         = 14;              // ATR(14,255,50,50,50,50,50,...) Period of averaging
input int    Signal_ATR_ThresholdPoints  = 255;             // ATR(14,255,50,50,50,50,50,...) ATR Threshold in Points
input int    Signal_ATR_Pattern_0        = 50;              // ATR(14,255,50,50,50,50,50,...) Pattern 0
input int    Signal_ATR_Pattern_1        = 50;              // ATR(14,255,50,50,50,50,50,...) Pattern 1
input int    Signal_ATR_Pattern_2        = 50;              // ATR(14,255,50,50,50,50,50,...) Pattern 2
input int    Signal_ATR_Pattern_3        = 50;              // ATR(14,255,50,50,50,50,50,...) Pattern 3
input int    Signal_ATR_Pattern_4        = 50;              // ATR(14,255,50,50,50,50,50,...) Pattern 4
input int    Signal_ATR_PatternsUsed     = 255;             // ATR(14,255,50,50,50,50,50,...) Patterns Used BitMap
input double Signal_ATR_Weight           = 1.0;             // ATR(14,255,50,50,50,50,50,...) Weight [0...1.0]
//--- inputs for trailing
input int    Trailing_ATR_PeriodMA       = 14;              // Period of averaging
input int    Trailing_ATR_ThresholdPoints = 255;            // ATR Threshold in Points
input int    Trailing_ATR_Pattern_0      = 50;              // Pattern 0
input int    Trailing_ATR_Pattern_1      = 50;              // Pattern 1
input int    Trailing_ATR_Pattern_2      = 50;              // Pattern 2
input int    Trailing_ATR_PatternsUsed   = 8;               // Patterns Used BitMap
input int    Trailing_ATR_StopLevel      = 30;              // Stop Loss trailing (in ATR multiples)
//--- inputs for money
input double Money_ATR_DecreaseFactor    = 3.0;             // Decrease factor
input double Money_ATR_Percent           = 10.0;            // Percent
input int    Money_ATR_ThresholdPoints   = 255;             // ATR Threshold in Points
input int    Money_ATR_Pattern_0         = 50;              // Pattern 0
input int    Money_ATR_Pattern_1         = 50;              // Pattern 1
input int    Money_ATR_Pattern_2         = 50;              // Pattern 2
input int    Money_ATR_PatternsUsed      = 8;               // Patterns Used BitMap
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
CExpert ExtExpert;
//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
{
//--- Initializing expert
   if(!ExtExpert.Init(Symbol(), Period(), Expert_EveryTick, Expert_MagicNumber))
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing expert");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Creating signal
   CExpertSignal *signal = new CExpertSignal;
   if(signal == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating signal");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//---
   ExtExpert.InitSignal(signal);
   signal.ThresholdOpen(Signal_ThresholdOpen);
   signal.ThresholdClose(Signal_ThresholdClose);
   signal.PriceLevel(Signal_PriceLevel);
   signal.StopLevel(Signal_StopLevel);
   signal.TakeLevel(Signal_TakeLevel);
   signal.Expiration(Signal_Expiration);
//--- Creating filter CSignalATR
   CSignalATR *filter0 = new CSignalATR;
   if(filter0 == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating filter0");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
   signal.AddFilter(filter0);
//--- Set filter parameters
   filter0.PeriodMA(Signal_ATR_PeriodMA);
   filter0.ThresholdPoints(Signal_ATR_ThresholdPoints);
   filter0.Pattern_0(Signal_ATR_Pattern_0);
   filter0.Pattern_1(Signal_ATR_Pattern_1);
   filter0.Pattern_2(Signal_ATR_Pattern_2);
   filter0.Pattern_3(Signal_ATR_Pattern_3);
   filter0.Pattern_4(Signal_ATR_Pattern_4);
filter0.PatternsUsed(Signal_ATR_PatternsUsed);
   filter0.Weight(Signal_ATR_Weight);
//--- Creation of trailing object
   CTrailingATR *trailing = new CTrailingATR;
   if(trailing == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert.InitTrailing(trailing))
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Set trailing parameters
   trailing.PeriodMA(Trailing_ATR_PeriodMA);
   trailing.Pattern_0(Trailing_ATR_Pattern_0);
   trailing.Pattern_1(Trailing_ATR_Pattern_1);
   trailing.Pattern_2(Trailing_ATR_Pattern_2);
   trailing.PatternsUsed(Trailing_ATR_PatternsUsed);
   trailing.StopLevel(Trailing_ATR_StopLevel);
//--- Creation of money object
   CMoneyATR *money = new CMoneyATR;
   if(money == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert.InitMoney(money))
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Set money parameters
   money.DecreaseFactor(Money_ATR_DecreaseFactor);
   money.Percent(Money_ATR_Percent);
   money.Pattern_0(Money_ATR_Pattern_0);
   money.Pattern_1(Money_ATR_Pattern_1);
   money.Pattern_2(Money_ATR_Pattern_2);
   money.PatternsUsed(Money_ATR_PatternsUsed);
//--- Check all trading objects parameters
   if(!ExtExpert.ValidationSettings())
   {  //--- failed
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Tuning of all necessary indicators
   if(!ExtExpert.InitIndicators())
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing indicators");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- ok
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{  ExtExpert.Deinit();
}
//+------------------------------------------------------------------+
//| "Tick" event handler function                                    |
//+------------------------------------------------------------------+
void OnTick()
{  ExtExpert.OnTick();
}
//+------------------------------------------------------------------+
//| "Trade" event handler function                                   |
//+------------------------------------------------------------------+
void OnTrade()
{  ExtExpert.OnTrade();
}
//+------------------------------------------------------------------+
//| "Timer" event handler function                                   |
//+------------------------------------------------------------------+
void OnTimer()
{  ExtExpert.OnTimer();
}
//+------------------------------------------------------------------+
